/*
 * Photo Image Print System
 * Copyright (C) Seiko Epson Corporation 2001-2008.
 *
 * This file is part of the `espmnt' program.
 *
 *  This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <stdio.h>
#include <stdlib.h>
#include <memory.h>
#include <getopt.h>
#include <ltdl.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <errno.h>
#include <string.h>

#include "pipsutil.h"
#include "utlcmd.h"

#include "pipscap-def.h"
#include "pipserr.h"
#include "ekpcom.h"

#ifndef N_
#define N_(word)	(word)
#endif

#ifdef DEBUG
#include "espdbg.h"
#endif
#ifndef DP
#define DP(e)
#endif

enum eMode{
	MODE_NONE = 0,
	MODE_INT,
};
 
enum eFNC{
	eFNC_UNKNOWN = 0,
	eFNC_HELP, 
	eFNC_PRM, 
	eFNC_EXEC	
};

#define NAME_LEN		(127)
#define FILENAME_LEN	(255)
#define RBUF_LEN		(1024)

#define MIN(a, b)	( ( (a)<(b) )? (a):(b) )

/* import liblary api */
static UC_CommandID			uc_CommandID;
static UC_EnumCmd			uc_EnumCmd;
static UC_EnumCmdNext		uc_EnumCmdNext;
static UC_GetCmdDesc		uc_GetCmdDesc;
static UC_MakeCommand		uc_MakeCommand;
static UC_GetNextCmd		uc_GetNextCmd;
static UC_CanExecute		uc_CanExecute;

static int ShowHelp(int mode);
static int CreateCommand(int cmd, const char *parameter, char **pCmd, int *pSize);
static int ExecCommand(const char *alias_name, int cmd, const char *parameter);
static int CheckStatus(int cmd);
static int UCErrConv(int ucErr);
static char* GetFifoPath (const char *alias_name);
static lt_dlhandle	load_library(const char* printername);
static int		 	unload_library(lt_dlhandle lib_handle);

char *Fifo_Path = NULL;
#define	FIFO_PATH	"/var/ekpd/ekplp0"
#define	LIB_CMD		"libpips-utl%s.so"

const char* usage = "Usage: espmnt [ -p <printername> ] -e <func> [ -v <value> ] .\n";
char PRINTER_PACKAGE[MAX_SIZE] = "pips-";


int
main (int argc, char *argv[])
{
	char printer[NAME_LEN+1];
    char* model_name = NULL;
    char* alias_name = NULL;
	char command[NAME_LEN+1];
	char values[NAME_LEN+1];
    int fnc = eFNC_UNKNOWN;
    int cmd = eUC_UNKNOWN;
    int mode = 0;
	lt_dlhandle hLib = NULL;
    int res = NO_ERROR;
    int c;
    int index = 0;
    struct option allopt[] =
        {
            { "exec", 		required_argument, 	NULL, 'e'},		/* fuction execute */
            { "func",		no_argument,		NULL, 'f'},		/* enum supported functions */
            { "num", 		required_argument, 	NULL, 'n'},		/* get function parameter num */
            { "printer", 	required_argument, 	NULL, 'p'},		/* specify printer name */
            { "value",		required_argument, 	NULL, 'v'},		/* specify function parameter value */
            { "help",		no_argument, 		&fnc, eFNC_HELP},		/* help */
            { "int",		no_argument, 		&mode, MODE_INT},		/* internal mode */
            {0,0,0,0}
        };

	printer[0] = command[0] = values[0] = '\0';
	
	while ((c = getopt_long (argc, argv, "e:fn:p:v:", allopt, &index)) != -1)
    {
       	DP(("opt = %d, index = %d, optarg = %s\n", c, index, optarg));
   
        switch (c)
        {
        case 'f':
        	fnc = eFNC_HELP;
        	break;
        	
        case 'n':	/* number of function parameter */
        	fnc = eFNC_PRM;
            if (strlen (optarg) < sizeof(command)-1) {
                strcpy(command, optarg);
            } else{
            	res = INVALID_OPTION_ERROR;
            }
            break;
            
        case 'e':	/* execution */
			fnc = eFNC_EXEC;
            if (strlen (optarg) < sizeof(command)-1) {
                strcpy(command, optarg);
            } else{
            	res = INVALID_OPTION_ERROR;
            }
            break;

        case 'p':	/* printer name */
            if (strlen (optarg) < sizeof(printer)-1) {
                strcpy(printer, optarg);
            } else{
            	res = INVALID_OPTION_ERROR;
            }
            break;

        case 'v':	/* parameter values */
            if (strlen (optarg) < sizeof(values)-1) {
                strcpy(values, optarg);
            } else{
            	res = INVALID_OPTION_ERROR;
            }
            break;
            
		case 0:
			break;
            
        default: /* ? */
        	fprintf(stderr, usage);
          	res = INVALID_OPTION_ERROR;
			break;
        }
      
		if(NO_ERROR != res){
			FILTER_ERROR_HANDLER (res, optarg);
		}
    }


	if( strlen(printer) == 0 ){
		if( !(model_name = GetDefaultPrinter(1)) ){
			char **model_list = NULL;
			model_list = GetModelListName (PNG_TYPE);
			if( *model_list ){
				model_name = strdup(*model_list);
				free(model_list);
			}else {
				FILTER_ERROR_HANDLER (PRINTER_NOT_FOUND, "decide printer faile.");
				return PRINTER_NOT_FOUND;
			}
		}
	} else{
		model_name = strdup(printer);
	}

	alias_name = GetAliasName(model_name);
	if (alias_name == NULL) {
		alias_name = strdup(model_name);
	}
	if(model_name)free(model_name);

	/* load printer command liblary */
	if(0 == res){
		hLib = load_library(alias_name);
		if( NULL == hLib ){
	        res = LIB_ERROR;
	    }
	}
	
DP(("%s fnc = %d\n", alias_name, fnc));
	if(NO_ERROR == res){
	/* text domain */
#ifdef ENABLE_NLS
	    bindtextdomain (PACKAGE, LOCALEDIR);
    	bind_textdomain_codeset (PACKAGE, "UTF-8");

   		strcat(PRINTER_PACKAGE, alias_name);
   	    bind_textdomain_codeset (PRINTER_PACKAGE, "UTF-8");
DP(("%s / %s\n", PACKAGE, PRINTER_PACKAGE));

	    textdomain (PACKAGE);
    	setlocale (LC_ALL, "");
#endif	

		switch(fnc){
		case eFNC_HELP:
			res = ShowHelp(mode);
			break;
		
		case eFNC_PRM:
			cmd = uc_CommandID(command);
			if(cmd != eUC_UNKNOWN){
				printf("%s\n", uc_GetCmdDesc(eUCF_PRM, cmd));
			} else{
				res = INVALID_OPTION_ERROR;
			}
			break;

		case eFNC_EXEC:
			cmd = uc_CommandID(command);
			if(cmd != eUC_UNKNOWN){
				do{
					res = ExecCommand(alias_name, cmd, values);
					if(NO_ERROR != res){
						break;
					}
					cmd = uc_GetNextCmd(cmd);
				} while(eUC_UNKNOWN != cmd);
			} else{
				res = INVALID_OPTION_ERROR;
			}
			break;
			
		default:
			res = NO_OPTION_ERROR;		
			break;
		}
	} 
	
	if(NO_ERROR != res){
		if(STATUS_ERROR == res){
			textdomain (PRINTER_PACKAGE);
			fprintf(stderr, "%s", _(uc_GetCmdDesc(eUCF_ERRMSG, cmd)));
			textdomain (PACKAGE);
		} else{
			FILTER_ERROR_HANDLER (res, NULL);
		}
	}	
	
	unload_library(hLib);
	
	if(Fifo_Path)free(Fifo_Path);
	
DP(("res = %d\n", res))	;
	return res;
}


int ShowHelp(int mode)
{
	int res = NO_ERROR;
	int cmd;
	
	if( MODE_INT == mode ){
		cmd = uc_EnumCmd();
		if(eUC_UNKNOWN != cmd){
			while( eUC_UNKNOWN != cmd ){
				printf("%s,", uc_GetCmdDesc(eUCF_ALIACE, cmd));
				cmd = uc_EnumCmdNext();
			}
		} else{
			res = LIB_INTERNAL_ERROR;
		}

	} else{
		printf("\n%s\n----- option -----\n", usage);
		printf(_("-p\t: Specifies the printer name.\n\t  If not specified, the default printer will be used.\n"));
		cmd = uc_EnumCmd();
		if(eUC_UNKNOWN != cmd){
			printf(_("-e\t: Specifies the utility to perform.\n\t  Specify one of the following values.\n"));

			textdomain (PRINTER_PACKAGE);
			while( eUC_UNKNOWN != cmd ){
				printf("\t  %s\t: %s\n", _(uc_GetCmdDesc(eUCF_ALIACE, cmd)), _(uc_GetCmdDesc(eUCF_DESC, cmd)));
				cmd = uc_EnumCmdNext();
			}
			textdomain (PACKAGE);
			
			printf(_("-v\t: Specifies the parameter of the utility to perform.\n\t  Set the following values for each utility.\n"));
			cmd = uc_EnumCmd();

			textdomain (PRINTER_PACKAGE);
			while( eUC_UNKNOWN != cmd ){
				if( uc_GetCmdDesc(eUCF_PRMHELP, cmd) ){
					printf("\t  %s\t: %s\n",
							_(uc_GetCmdDesc(eUCF_ALIACE, cmd)),
							_(uc_GetCmdDesc(eUCF_PRMHELP, cmd)));
				}
				cmd = uc_EnumCmdNext();
			}
			textdomain (PACKAGE);
		} else{
			printf(_("-e\t: Specifies the utility to perform.\n\t  Specify one of the following values.\n"));
			printf(_("-v\t: Specifies the parameter of the utility to perform.\n\t  Set the following values for each utility.\n"));
			printf(_("\nplease specify priter name.\n [ espmnt -p <printername> --help ]\n"));
		}
	}
	
	return res;
}


int CreateCommand(int cmd, const char *parameter, char **pCmd, int *pSize)
{
	int res = NO_ERROR;

	res = uc_MakeCommand(cmd, NULL, pSize, parameter);
	if(UCE_OK != res){
		return UCErrConv(res);
	}
    *pCmd = malloc(*pSize);
    if( !*pCmd ){
    	return MEMORY_ERROR;
    }
   
	res = uc_MakeCommand(cmd, *pCmd, pSize, parameter);
	DP(("MakeCommand = %d / size = %d \n", res, *pSize));
	if(0 != res){
		free(*pCmd);
		*pCmd = NULL;
		return UCErrConv(res);
	}

	return NO_ERROR;
}


int ExecCommand(const char *alias_name, int cmd, const char *parameter)
{
	int res = NO_ERROR;
	char *pCmd = NULL;
	char *p = NULL;
	int nCmdSize = 0;
    int fd;			/* Parallel device */
    int wbytes;		/* Number of bytes written */
	
	/* check printer status */
	if( NO_ERROR != (res = CheckStatus(cmd)) ){
		return res;
	}

    /* Make Command packet */
	res = CreateCommand(cmd, parameter, &pCmd, &nCmdSize);
	if(UCE_OK != res){
		return res;
	}

#ifdef DEBUG
DP(("sleep 5\n"));
sleep(5);
{
	FILE* f;
	char fn[32];
	sprintf(fn, "cmd%d.prn", cmd);
	f = fopen(fn, "wb");
	if(f){
		fwrite(pCmd, sizeof(char), nCmdSize, f);

        fflush(f);
	    fclose(f);
	}
}
return 0;
#endif
	if( !Fifo_Path ){
		Fifo_Path = GetFifoPath(alias_name);
DP(("fifo : %s\n", Fifo_Path));
		if( !Fifo_Path ){
			Fifo_Path = strdup(FIFO_PATH);
			return RESOURCE_VALUES_ERROR;
		}
	}

	do {
	    if ((fd = open (Fifo_Path, O_WRONLY | O_EXCL)) == -1) {
	        if (errno == EBUSY) {
	            err_msg (MSGTYPE_INFO, "PIPS port busy; will retry in 10 seconds...\n");
	            sleep (10);
	        } else {
	            fprintf(stderr, "Unable to open PIPS port device file. %s\n",
	                       strerror (errno));
	            res = COMMUNICATION_ERROR;
	            break;
	        }
	    }
	} while (fd < 0 && res == NO_ERROR);

	p = pCmd;
    while (nCmdSize > 0 && res == NO_ERROR) {
        if ((wbytes = write (fd, p, nCmdSize)) < 0){
            if (errno == ENOTTY){
                wbytes = write (fd, p, nCmdSize);
			}
		}

        if (wbytes < 0) {
            err_msg (MSGTYPE_ERROR, "Unable to send print file to printer.");
            break;
        }
        nCmdSize -= wbytes;
        p += wbytes;
	}

	free(pCmd);

	return res;
}


int UCErrConv(int ucErr)
{
	switch(ucErr){
	case UCE_OK:
		return NO_ERROR;
	case UCE_MEMORY:
		return MEMORY_ERROR;
	case UCE_INVALID_PARAM:
		return INVALID_OPTION_ERROR;
	case UCE_RESOURCE:
		return NO_RESOURCE_ERROR;
	default:
		return UNEXPECTED_ERROR;
	}
}


int CheckStatus(int cmd)
{
	int res = NO_ERROR;
	char pCmd[] = {'s', 't', 0x01, 0x00, 0x01};
	int nCmdSize = sizeof(pCmd);
	char reply[RBUF_LEN+1];
	int replySize = RBUF_LEN;
	
	memset(reply, 0, sizeof(reply));

	res = sock_open();
	if(0 == res){
		res = sock_write(pCmd, &nCmdSize);
		if(0 == res){
			res = sock_read(reply, &replySize);
		}
		sock_close();
	}
DP(("ST= %s\n", reply));

	if( !uc_CanExecute(cmd, reply) ){
		return STATUS_ERROR;
	}	

	return res;
}


lt_dlhandle load_library(const char* printername)
{
    int err = 0;
	char lib_path[FILENAME_LEN+1];
    lt_dlhandle lib_handle = NULL;
    
//    sprintf(lib_path,"/usr/lib/libpips-utl%s.so", printername);
    sprintf(lib_path, LIB_CMD, printername);
    
    err = lt_dlinit();
    if (!err)
    {
        lib_handle = lt_dlopen (lib_path);
    }
    
    if( !lib_handle ){
    	FILTER_ERROR_HANDLER (LIB_ERROR, lib_path);
    	return NULL;
	}
  
    uc_CommandID   		= (UC_CommandID)		lt_dlsym(lib_handle, "ucCommandID");
    uc_EnumCmd  		= (UC_EnumCmd)			lt_dlsym(lib_handle, "ucEnumCmd");
    uc_EnumCmdNext  	= (UC_EnumCmdNext)		lt_dlsym(lib_handle, "ucEnumCmdNext");
    uc_GetCmdDesc		= (UC_GetCmdDesc)		lt_dlsym(lib_handle, "ucGetCmdDesc");
    uc_MakeCommand		= (UC_MakeCommand)		lt_dlsym(lib_handle, "ucMakeCommand");
    uc_GetNextCmd		= (UC_GetNextCmd)		lt_dlsym(lib_handle, "ucGetNextCmd");
    uc_CanExecute		= (UC_CanExecute)		lt_dlsym(lib_handle, "ucCanExecute");
    
DP(("%x / %x / %x / %x / %x / %x \n",
		uc_CommandID, uc_EnumCmd, uc_EnumCmdNext, uc_GetCmdDesc, uc_MakeCommand, uc_GetNextCmd));
        
    if (   !uc_CommandID
    	|| !uc_EnumCmd
    	|| !uc_EnumCmdNext    	
        || !uc_GetCmdDesc
        || !uc_MakeCommand
        || !uc_GetNextCmd
        || !uc_CanExecute)
    {
        unload_library(lib_handle);
        lib_handle = NULL;
    }

    return lib_handle;
};


int unload_library(lt_dlhandle lib_handle)
{
	if( lib_handle ){
	    if( lt_dlclose (lib_handle) == -1 ){
    	    return LIB_ERROR;
	    }
	}
	
    lt_dlexit();
    
    return 0;
};


char* GetFifoPath (const char *alias_name)
{
    xmlDocPtr doc = NULL;
    xmlNodePtr ndEnt, ndElm;
    char *tmpAlies = NULL;
    char *dmyPath = NULL;
    int found = 0;

    doc = xmlParseFile (PIPS_EKPD_RC_PATH);
    ndEnt = xmlDocGetRootElement (doc);
    
    /* Go to the next child */
    ndEnt = ndEnt->xmlChildrenNode;
    while( ndEnt != NULL ){
	    ndElm = ndEnt->xmlChildrenNode;
	    if( dmyPath ){
	    	free(dmyPath);
	    	dmyPath = NULL;
	    }
	    
	    found = 0;
	    while( ndElm != NULL && !(found && dmyPath) ){
DP(("name : %s\n", ndElm->name));
	    	if( !strcmp((char*)ndElm->name, "aliasname") ){
	    		tmpAlies = (char*)xmlNodeListGetString(doc, ndElm->xmlChildrenNode, 1);
	    		if(!strcmp(tmpAlies, alias_name)) found++;
	    		free(tmpAlies);
	    	} else if( !strcmp((char*)ndElm->name, "dummypath") ){
	    		dmyPath = (char*)xmlNodeListGetString(doc, ndElm->xmlChildrenNode, 1);
	    	}
	    
	    	ndElm = ndElm->next;
	    }
	    
	    if( found && dmyPath ){
	    	break;
	    }
		ndEnt = ndEnt->next;
	}
    
    xmlFreeDoc(doc);

    return dmyPath;
}

