/*
 * Photo Image Print System
 * Copyright (C) 2000-2005 EPSON AVASYS Corporation.
 * Copyright (C) Seiko Epson Corporation 2000-2008.
 *
 *  This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * As a special exception, EPSON AVASYS Corporation gives permission to
 * link the code of this program with libraries which are covered by
 * the EPSON AVASYS Public License and distribute their linked
 * combinations.  You must obey the GNU General Public License in all
 * respects for all of the code used other than the libraries which
 * are covered by EPSON AVASYS Public License.
 */
#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <stdio.h>
#include <errno.h>
#include <stdlib.h>
#include <string.h>
#include "pipserr.h"

#ifndef N_
#define N_(word)	(word)
#endif

/* global  */
static char err_pname[256] = "";

/* static functions */
static void err_doit (enum Message_Type, int, const char *, va_list);

void
pips_error(int ecode, char *err_data)
{
    char *message;

    if (ecode == NO_ERROR)
        return;

    switch(ecode)
    {
    /* Option Error */
    case NO_OPTION_ERROR:
        message = N_("Not enough option.\n");
        break;

    case OPTION_LESS_ERROR:
        message = N_("Option value is not exist.\n");
        break;

    case OPTION_VALUES_ERROR:
        message = N_(" Option value is incorrect.\n");
        break;

    case DUB_FILENAME_ERROR:
        message = N_("Too many input files.\n");
        break;

    case COMBINE_OPTION_ERROR:
        message = N_("Illegal option combination.\n");
        break;

    case INVALID_OPTION_ERROR:
        message = N_("Invalid option.\n");
        break;

    case INVALID_FILTER_ERROR:
        message = N_("Invalid filter type.\n");
        break;
    /* Resource File Error */
    case NO_RESOURCE_ERROR:
        message = N_("The resource file is not exist.\n");
        break;

    case NOT_WELL_FORM_ERROR:
        message = N_("The xml file is not well-form.\n");
        break;

    case RESOURCE_FORMAT_ERROR:
        message = N_("The resource file format is incorrect.\n");
        break;

    case RESOURCE_OPTION_ERROR:
        message = N_("Resource option key is incorrect.\n");
        break;

    case RESOURCE_VALUES_ERROR:
        message = N_("Resource value is incorrect.\n");
        break;

    case RESOURCE_ATTRIBUTE_ERROR:
        message = N_("Resource attribute is incorrect.\n");
        break;

    case RESOURCE_EMPTY_ERROR:
        message = N_("The resource file is empty.\n");
        break;

    case PARSER_ERROR:
        message = N_("The xml file was not parsed successfully.\n");
        break;
    /* File/Image Error */
    case IMAGE_CONVERT_ERROR:
        message = N_("Format conversion error.\n");
        break;

    case IMAGE_NO_SUPPORT_ERROR:
        message = N_("The input format is not supported.\n");
        break;

    case IMAGE_ATTRIBUTE_ERROR:
        message = N_("Illegal attribute value.\n");
        break;

    case FILE_NOT_FOUND_ERROR:
        message = N_("File not found.\n");
        break;

    case FILE_ACCESS_ERROR:
        message = N_("File accessing error.\n");
        break;
    /* Library Error */
    case LIB_PNG_ERROR:
        message = N_("The input file is corrupt.\n");
        break;

    case LIB_ERROR:
        message = N_("Can not find library.\n");
        break;

    case LIB_INTERNAL_ERROR:
        message = N_("Library internal error.\n");
        break;

    case NOT_EXIST_HANDLE_ERROR:
        message = N_("The handle does not exist.\n");
        break;

    case MEMORY_ERROR:
        message = N_("Out of memory.\n");
        break;

    case UNEXPECTED_ERROR:
        message = N_("Unexpected error.\n");
        break;

    case ABNORMAL_END:
        message = N_("It can not continue with PIPS.\n");
        break;

    case PRINTER_NOT_FOUND:
        message = N_("Printer not found.\n");
        break;

    case PIPSRC_NOT_FOUND:
        message = N_("The /etc/pipsrc file was not found!\nThe default mode is loading...\n");
        break;

    case DEF_PRINTER_NOT_FOUND:
        message = N_("Default printer not found. Find the next printer...\n");
        break;

    case LPR_NOT_EXECUTED:
        message = N_("This utility is only for lpr.\nLpd is not executed or other spoolers are executed.\n");
        break;

    case COMMUNICATION_ERROR:
        message = N_("Communication Error.\n");
        break;
        
    case UNSUPPORTED_ERROR:
        message = N_("Printer don't support it.\n");
        break;
        
    case STATUS_ERROR:
        message = N_("Unable to execute by status of printer.\n");
        break;

    default:
        message = N_("Unknown error occurred.\n");
        break;

    }

    if (ecode != PIPSRC_NOT_FOUND) /* Waring */
        fprintf (stderr, N_("PIPS Error: "));
    else
        fprintf (stderr, N_("PIPS Warning: "));

    if (err_data != NULL && (strlen (err_data) > 0))
        fprintf (stderr, "%s : ", err_data);

    fprintf (stderr, "%s", _(message));

    if (stderr)
        fflush (stderr);
}

void
err_init (const char *name)
{
    if (name && strlen (name) < 256)
        strcpy (err_pname, name);

    return;
}

void
err_msg (enum Message_Type type, const char *fmt, ...)
{
    va_list ap;
    va_start (ap, fmt);
    err_doit (type, 0, fmt, ap);
    va_end (ap);

    return;
}

void
err_fatal (const char *fmt, ...)
{
    va_list ap;
    va_start (ap, fmt);
    err_doit (MSGTYPE_ERROR, 0, fmt, ap);
    va_end (ap);
    exit (1);
}

void
err_system (const char *fmt, ...)
{
    int e;
    va_list ap;
    e = errno;
    va_start (ap, fmt);
    err_doit (MSGTYPE_ERROR, e, fmt, ap);
    va_end (ap);
    exit (1);
}

static void
err_doit (enum Message_Type type, int e, const char *fmt, va_list ap)
{
    if (err_pname[0] != '\0')
        fprintf (stderr, "%s : ", err_pname);

    if (type == MSGTYPE_ERROR)
        fprintf (stderr, N_("**** ERROR **** : "));
    else if (type == MSGTYPE_WARNING)
        fprintf (stderr, N_("**** WARNING **** : "));
    else if (type == MSGTYPE_INFO)
        fprintf (stderr, N_("**** INFO **** : "));

    vfprintf (stderr, fmt, ap);
    if (e)
        fprintf (stderr, " : %s", strerror (e));

    fprintf (stderr, "\n");
    fflush (stderr);

    return;
}
