/*
 * Photo Image Print System
 * Copyright (C) 2001-2005 EPSON AVASYS Corporation.
 * Copyright (C) Seiko Epson Corporation 2001-2008.
 *
 * This file is part of the `ekpstm' program.
 *
 *  This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <unistd.h>
#include <getopt.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <ltdl.h>
#include <locale.h>
#include "ekpcom.h"
#include "setup.h"
#include "pipserr.h"
#include "interface.h"
#include "support.h"
#include "stm.h"

#define DOWN_TIMEOUT 60

typedef void *CapHandle;
int (*OpenCap) (CapHandle *, char*, int, char[]);
int (*CloseCap) (CapHandle);
int (*ReferCapContext)  (CapHandle, CapContextList*);

static void destroy (GtkWidget*, gpointer);
static int GetStatusInfor (char *model_name);

CapContextList p_caplist;

int
main (int argc, char *argv[])
{
    GtkWidget *stm_window;
    int c;
    int index = 0, ret_code = 0;
    char* printer = NULL;
    char* pixmap_dir = NULL;
    char* model_name = NULL;
    char* usage = "Usage: -P [printer] or Not use argument for default printer.\n";

    struct option allopt[] =
        {
            { "auto", 0, 0, 'a'},
            { "printer",1, 0, 'P'},
            {0,0,0,0}
        };

    gtk_init (&argc, &argv);
    opterr = 0;
    /* Getopt */
    auto_down_set (0);
    while ((c = getopt_long (argc, argv, "aP:", allopt, &index)) != -1)
    {
        switch (c)
        {
        case 'a':
            auto_down_set (DOWN_TIMEOUT);
            break;
        case 'P':
            if (strlen (optarg) < 128)
            {
                printer= (char*)malloc(strlen(optarg)+1);
                memcpy (printer, optarg, strlen(optarg) + 1);
            }
        break;
        default:
            if (!strcmp (argv[optind-1],"-P") && !optarg)
               FILTER_ERROR_HANDLER (OPTION_VALUES_ERROR, "-P");
            else
               FILTER_ERROR_HANDLER (INVALID_OPTION_ERROR, argv[1]);

            fprintf(stderr, usage);
            return 1;
        }
    }

    if (optind < argc)
    {
        FILTER_ERROR_HANDLER (INVALID_OPTION_ERROR, argv[optind]);
        fprintf(stderr, usage);
        return 1;
    }

    /*
     * The following code was added by Glade to create one of each component
     * (except popup menus), just so that you see something after building
     * the project. Delete any components that you don't want shown initially.
     */

    /* exec when printer is not default printer */
    if(printer == NULL)
    {
	    if (!(model_name = GetDefaultPrinter(1)))
	    {
		    FILTER_ERROR_HANDLER (DEF_PRINTER_NOT_FOUND, NULL);
		    char **model_list = NULL;
		    model_list = GetModelListName (PNG_TYPE);
		    if (*model_list)
			    model_name = *model_list;
		    else
		    {
		    FILTER_ERROR_HANDLER (PRINTER_NOT_FOUND, NULL);
		    return 1;
		    }
	    }
    } else {
	    model_name = printer;
    }
	model_name = GetAliasName(model_name);
	if (model_name == NULL) {
		FILTER_ERROR_HANDLER (PRINTER_NOT_FOUND, NULL);
		return 1;
	}    

    pixmap_dir = (char*)malloc(strlen(PIPS_PRINTER_DIR)+strlen(model_name)+12);
    sprintf(pixmap_dir,PIPS_PRINTER_DIR"/%s/pixmaps",model_name);
    add_pixmap_directory(pixmap_dir);
    free(pixmap_dir);
    pixmap_dir = NULL;

    if ((ret_code = GetStatusInfor (model_name)))
    {
        FILTER_ERROR_HANDLER (ret_code, NULL);
        return 1;
    }

    bindtextdomain (PACKAGE, LOCALEDIR);
    bind_textdomain_codeset (PACKAGE, "UTF-8");
    textdomain (PACKAGE);
    setlocale (LC_ALL, "");

    stm_window = create_stm_window ();
    gtk_widget_show (stm_window);
    g_signal_connect (G_OBJECT (stm_window), "destroy", G_CALLBACK (destroy), NULL);
    /* socket �³ */
    sock_open ();
    init_stm (stm_window,printer);
    gtk_main ();

    ekpstm_pixbuf_list_free();

    free(printer);
    printer = NULL;

    return 0;
}

static void
destroy (GtkWidget *widget, gpointer data)
{
    end_stm ();
    return;
}

static int
GetStatusInfor (char *model_name)
{
    char cap_path[] = LIB_CAP_PATH, err_data[MAX_SIZE];
    int ret_code = 0;
    void *libcap_handle = 0;
    CapHandle cap_handle;

    if (lt_dlinit())
        return LIB_ERROR;

    libcap_handle = lt_dlopen (cap_path);
    if (libcap_handle == NULL)
        return LIB_ERROR;
    else
    {
        if (!(OpenCap = lt_dlsym (libcap_handle, "OpenCap"))   ||
            !(CloseCap = lt_dlsym (libcap_handle, "CloseCap")) ||
            !(ReferCapContext = lt_dlsym (libcap_handle, "ReferCapContext")))
        {
            lt_dlclose (libcap_handle);
            libcap_handle = 0;
        }
    }

    if ((ret_code = (*OpenCap) (&cap_handle, model_name, STM_TYPE, err_data)))
        return ret_code;
    if ((ret_code = (*ReferCapContext) (cap_handle, &p_caplist)))
        return ret_code;
    if ((ret_code = (*CloseCap) (cap_handle)))
        return ret_code;
    if (lt_dlclose (libcap_handle) == -1)
        return LIB_ERROR;

    lt_dlexit();

    return 0;
}





